package eu.osci;

import javax.xml.namespace.QName;

import static eu.osci.OSCIConstants.NAMESPACE_OSCI20;


/**
 * The OSCI header tags.<br>
 * Copyright  2010 Freie Hansestadt Bremen. <br>
 * Created by bremen online services Entwicklungs- und Betriebsgesellschaft mbH & Co. KG. <br>
 * This library (Starter Kit) can be used by anybody according to the Bremer Lizenz fr freie
 * Softwarebibliotheken. The license terms can be viewed at www.osci.de or can be ordered at the
 * OSCI-Leitstelle (Die Senatorin fr Finanzen, Freie Hansestadt Bremen, Postfach 10 15 40, 28015 Bremen).
 * 
 * @author A. Nagel
 */

public enum OSCIHeaderTag
{
  /**
   * The QName for the OSCI 'MsgTimeStamps' element.
   */
  MSGTIMESTAMPS(new QName(NAMESPACE_OSCI20, "MsgTimeStamps")),

  /**
   * The QName for the OSCI 'DeliveryReceiptDemand' element.
   */
  DELIVERY_DEMAND(new QName(NAMESPACE_OSCI20, "DeliveryReceiptDemand")),

  /**
   * The QName for the OSCI 'ReceptionReceiptDemand' element.
   */
  RECEIPT_DEMAND(new QName(NAMESPACE_OSCI20, "ReceptionReceiptDemand")),

  /**
   * The QName for the OSCI 'FetchedNotificationDemand' element.
   */
  FETCHED_NOTIFICATION_DEMAND(new QName(NAMESPACE_OSCI20, "FetchedNotificationDemand")),

  /**
   * The QName for the OSCI 'X509TokenContainer' element.
   */
  X509CONTAINER(new QName(NAMESPACE_OSCI20, "X509TokenContainer")),

  /**
   * The QName for the OSCI 'TypeOfBusinessScenario' element.
   */
  TYPEOFBUSINESS(new QName(NAMESPACE_OSCI20, "TypeOfBusinessScenario")),

  /**
   * The QName for the OSCI 'MsgBoxResponse' element.
   */
  MSGBOX_RESPONSE(new QName(NAMESPACE_OSCI20, "MsgBoxResponse")),

  /**
   * The QName for the OSCI 'DeliveryReceipt' element.
   */
  DELIVERY_RECEIPT(new QName(NAMESPACE_OSCI20, "DeliveryReceipt"));

  // The following items are not used yet
  /**
   * The QName for the OSCI 'MsgBoxFetchRequest' element.
   */
  // FETCH_REQUEST(new QName(NAMESPACE_OSCI20, "MsgBoxFetchRequest"));
  /**
   * The QName for the OSCI 'FetchNotifications' element.
   */
  // FETCH_NOTIFICATION(new QName(NAMESPACE_OSCI20, "FetchNotifications"));
  /**
   * The QName for the OSCI 'ReceptionReceipt' element.
   */
  // RECEPTION_RECEIPT(new QName(NAMESPACE_OSCI20, "ReceptionReceipt"));
  private QName qname;

  private OSCIHeaderTag(QName _qname)
  {
    qname = _qname;
  }

  /**
   * @return The property value.
   */
  public QName getValue()
  {
    return qname;
  }


}
