package eu.osci.messagetypes;


import java.io.IOException;
import java.security.AccessController;
import java.util.GregorianCalendar;
import java.util.LinkedList;
import java.util.List;
import java.util.logging.Logger;

import javax.security.auth.Subject;
import javax.security.auth.callback.Callback;
import javax.security.auth.callback.CallbackHandler;
import javax.security.auth.callback.UnsupportedCallbackException;

import org.w3c.dom.Element;

import com.sun.xml.wss.impl.callback.SAMLCallback;
import com.sun.xml.wss.saml.Assertion;
import com.sun.xml.wss.saml.AuthnContext;
import com.sun.xml.wss.saml.AuthnStatement;
import com.sun.xml.wss.saml.Conditions;
import com.sun.xml.wss.saml.NameID;
import com.sun.xml.wss.saml.SAMLAssertionFactory;
import com.sun.xml.wss.saml.SubjectConfirmation;


/**
 * 
 * @author R. Lindemann
 */
public class SamlCallbackHandler implements CallbackHandler {
	/** Logger to log the messages. */
	public static Logger LOG = Logger.getLogger(SamlCallbackHandler.class
			.getName());

	public static final String SAML_QA_LEVEL = "saml.citizenQAAlevel";
	public static String SAML_SUBEJCT_SURENAME = "saml.subject.surname";
	public static String SAML_SUBEJCT_GIVENNAME = "givenName";
	public static String SAML_SUBEJCT_EIDENTIFIER = "eIdentifier";
	public static String SAML_SUBEJCT_DATE_OF_BIRTH = "dateOfBirth";

	private int count = 0;
	private UnsupportedCallbackException unsupported = new UnsupportedCallbackException(
			null, "Unsupported Callback Type Encountered");

	public SamlCallbackHandler() {
	}

	public void handle(Callback[] callbacks) throws IOException,
			UnsupportedCallbackException {
//		Subject sub = Subject.getSubject(AccessController.getContext());
		LOG.severe("######################## SAML call.");
//		if (sub == null) {
//			// TODO RL: what shall we do?
//			throw new RuntimeException("No context found.");
//		}
//		SenderConfigration senderConfig = sub
//				.getPublicCredentials(SenderConfigration.class).iterator()
//				.next();
//		LOG.info("Name of sender: " + senderConfig.getSurname());
		for (int i = 0; i < callbacks.length; i++) {
			if (callbacks[i] instanceof SAMLCallback) {
				try {
					SAMLCallback samlCallback = (SAMLCallback) callbacks[i];
					if (samlCallback.getConfirmationMethod().equals(
							SAMLCallback.SV_ASSERTION_TYPE)
							&& count == 0) {
						samlCallback
								.setAssertionElement(createSenderVouchesSAMLAssertion(
										samlCallback));
					}
				} catch (Exception ex) {
					ex.printStackTrace();
				}
			} else {
				throw unsupported;
			}
		}
	}

	private Element createSenderVouchesSAMLAssertion(SAMLCallback samlCallback) {
		Assertion assertion = null;
		try {
			String issuerName ="Issuer";// getConfiguration().getGatewayAddress();
			// create the assertion id
			String aID = String.valueOf(System.currentTimeMillis());
			GregorianCalendar c = new GregorianCalendar();
			long beforeTime = c.getTimeInMillis();
			// roll the time by one hour
			long offsetHours = 60 * 60 * 1000;
			c.setTimeInMillis(beforeTime - offsetHours);
			GregorianCalendar before = (GregorianCalendar) c.clone();
			c = new GregorianCalendar();
			long afterTime = c.getTimeInMillis();
			c.setTimeInMillis(afterTime + offsetHours);
			GregorianCalendar after = (GregorianCalendar) c.clone();
			GregorianCalendar issueInstant = new GregorianCalendar();
			// statements
			List<Object> statements = new LinkedList<Object>();
			SAMLAssertionFactory factory = SAMLAssertionFactory
					.newInstance(SAMLAssertionFactory.SAML2_0);
			NameID issuerNameID = factory.createNameID(issuerName, null,
					"urn:oasis:names:tc:SAML:2.0:nameid-format:entity");
			SubjectConfirmation scf = factory.createSubjectConfirmation(
					issuerNameID,
					"urn:oasis:names:tc:SAML:2.0:cm:sender-vouches");

			NameID senderNameID = factory.createNameID("SenderIDname",
					issuerName,
					"urn:oasis:names:tc:SAML:2.0:nameid-format:unspecified");
			com.sun.xml.wss.saml.Subject subj = factory.createSubject(
					senderNameID, scf);
			AuthnContext ctx = factory.createAuthnContext(
					"urn:oasis:names:tc:SAML:2.0:ac:classes:X509", null);
			final AuthnStatement statement = factory.createAuthnStatement(
					issueInstant, null, ctx, null, null);
			statements.add(statement);
//			statements.add(createAttributeStatementList(factory));

			Conditions conditions = factory.createConditions(before, after,
					null, null, null, null);
			assertion = factory.createAssertion(aID, issuerNameID,
					issueInstant, conditions, null, subj, statements);
//			sign(assertion);
			return assertion.toElement(null);
		} catch (Exception e) {
			throw new RuntimeException(e);
		}
	}
//
//	public void sign(Assertion assertion) throws SAMLException {
//
//		XMLSignatureFactory fac = XMLSignatureFactory.getInstance();
//		try {
//
//			PrivateKey privKey = Configuration.getConfiguration()
//					.getSignatureKey();
//			X509Certificate cert = Configuration.getConfiguration()
//					.getSignatureCertificate();
//			Transform transformEnv = fac.newTransform(Transform.ENVELOPED,
//					(TransformParameterSpec) null);
//			XMLSignatureHelper.createSignature(
//					Arrays.asList("#" + assertion.getID()), privKey, cert,
//					assertion.toElement(null), transformEnv);
//			// Go on without escalation so far
//		} catch (GeneralSecurityException ex) {
//			LOG.log(Level.SEVERE, "Error signing the SAML token.", ex);
//		} catch (MarshalException ex) {
//			LOG.log(Level.SEVERE, "Error signing the SAML token.", ex);
//		} catch (XMLSignatureException ex) {
//			LOG.log(Level.SEVERE, "Error signing the SAML token.", ex);
//		} catch (XWSSecurityException ex) {
//			LOG.log(Level.SEVERE, "Error signing the SAML token.", ex);
//		} catch (IOException ex) {
//			LOG.log(Level.SEVERE, "Error signing the SAML token.", ex);
//		}
//	}

//	private AttributeStatement createAttributeStatementList(
//			SAMLAssertionFactory samlAssertFactory,
//			SenderConfigration senderConfig) throws SAMLException {
//		List<Attribute> attrs = new ArrayList<Attribute>();
//		// TODO this values should be set dynamic
//		// creates the AttributeStatement QAAlevel
//		attrs.add(createAttributeStatement(samlAssertFactory,
//				"http://www.stork.gov.eu/1.0/citizenQAAlevel", ""
//						+ senderConfig.getCitizenQAAlevel()));
//		// creates the AttributeStatement Surname
//		if (senderConfig.getSurname() != null)
//			attrs.add(createAttributeStatement(samlAssertFactory,
//					"http://www.stork.gov.eu/1.0/surname",
//					senderConfig.getSurname()));
//		// creates the AttributeStatement givenname
//		if (senderConfig.getGivenName() != null)
//			attrs.add(createAttributeStatement(samlAssertFactory,
//					"http://www.stork.gov.eu/1.0/givenName",
//					senderConfig.getGivenName()));
//		// creates the AttributeStatement eIDentifier
//		if (senderConfig.geteIdentifier() != null)
//			attrs.add(createAttributeStatement(samlAssertFactory,
//					"http://www.stork.gov.eu/1.0/eIdentifier",
//					senderConfig.geteIdentifier()));
//		// creates the AttributeStatement date of birth
//		if (senderConfig.getDateOfBirth() != null)
//			attrs.add(createAttributeStatement(samlAssertFactory,
//					"http://www.stork.gov.eu/1.0/dateOfBirth",
//					getDateAsString(senderConfig.getDateOfBirth())));
//		AttributeStatement statement = samlAssertFactory
//				.createAttributeStatement(attrs);
//		return statement;
//
//	}
//
//	private Configuration getConfiguration() {
//		try {
//			return Configuration.getConfiguration();
//		} catch (IOException ex) {
//			throw new RuntimeException("Error get the configuration", ex);
//		} catch (GeneralSecurityException ex) {
//			throw new RuntimeException("Error get the configuration", ex);
//		}
//	}
//
//	private String getDateAsString(Date date) {
//		GregorianCalendar cal = new GregorianCalendar();
//		cal.setTime(date);
//		String ret = cal.get(Calendar.YEAR) + "-" + cal.get(Calendar.MONTH)
//				+ "-" + cal.get(Calendar.DAY_OF_MONTH);
//		return ret;
//
//	}
//
//	private Attribute createAttributeStatement(
//			SAMLAssertionFactory samlAssertFactory, String _key,
//			String... _values) throws SAMLException {
//		List<String> values = new ArrayList<String>();
//		for (String value : _values) {
//			values.add(value);
//		}
//		Attribute attr = samlAssertFactory.createAttribute(_key,
//				"urn:oasis:names:tc:SAML:2.0:attrname-format:uri", values);
//		return attr;
//	}

}
